@@ -11,9 +11,8 @@ from typing import List
 from tqdm import tqdm
 import torch
 from torch.utils.data import Dataset
-from torch.utils.data.dataloader import DataLoader, default_collate
+from torch.utils.data.dataloader import default_collate
 from torchtext.data.utils import get_tokenizer
-import pytorch_lightning as pl
 
 # own
 from multimodal.features import get_features
@@ -21,7 +20,6 @@ from multimodal.datasets import vqa_utils
 from multimodal import DEFAULT_DATA_DIR
 from multimodal.utils import download_and_unzip
 from multimodal.datasets.vqa_utils import EvalAIAnswerProcessor
-from multimodal.text import WordEmbedding
 
 
 class AbstractVQA(Dataset):
@@ -33,6 +31,30 @@ class AbstractVQA(Dataset):
 
 
 class VQA(AbstractVQA):
+    """
+    Pytorch Dataset implementation for the VQA v1 dataset (visual question answering). 
+    See https://visualqa.org/ for more details about it.
+    
+    When this class is instanciated, data will be downloaded in the directory specified by the ``dir_data`` parameter.
+    Pre-processing of questions and answers will take several minutes.
+
+    When the ``features`` argument is specified, visual features will be downloaded as well. About 60Go will be 
+    necessary for downloading and extracting features.
+
+    Args:
+        dir_data (str): dir for the multimodal cache (data will be downloaded in a vqa2/ folder inside this directory
+        features (str|object): which visual features should be used. Choices: ``coco-bottomup`` or ``coco-bottomup-36``
+            You can also give directly the feature instance.
+        split (str): Which t [``train``, ``val``, ``test``]
+        dir_features (str): directory to download features. If None, defaults to $dir_data/features
+        label (str): either `multilabel`, or `best`. For `multilabel`, GT scores for questions are
+            given by the score they are assigned by the VQA evaluation. 
+            If `best`, GT is the label of the top answer.
+        tokenize_questions (bool): If True, preprocessing will tokenize questions into tokens.
+            The tokens are stored in item["question_tokens"].
+        load (bool): default `True`. If false, then the questions annotations and questions will not be loaded
+            in memory. This is useful if you want only to download and process the data.
+    """
 
     SPLITS = ["train", "val", "test", "test-dev"]
 
@@ -69,18 +91,8 @@ class VQA(AbstractVQA):
         dir_features=None,
         label="multilabel",
         tokenize_questions=False,
+        load=True,
     ):
-        """
-        dir_data: dir for the multimodal cache (data will be downloaded in a vqa2/ folder inside this directory
-        features: which visual features should be used. Choices: coco-bottomup or coco-bottomup-36
-        split: in [train, val, test]
-        dir_features: directory to download features. If None, defaults to $dir_data/features
-        label: either `multilabel`, or `best`. For `multilabel`, GT scores for questions are
-            given by the score they are assigned by the VQA evaluation. 
-            If `best`, GT is the label of the top answer.
-        tokenize_questions: If True, preprocessing will tokenize questions into tokens.
-            The tokens are stored in item["question_tokens"].
-        """
         self.dir_data = dir_data
         if self.dir_data is None:
             self.dir_data = DEFAULT_DATA_DIR
@@ -139,34 +151,38 @@ class VQA(AbstractVQA):
             self.dir_dataset, f"aid_to_ans-{self.min_ans_occ}.json"
         )
 
-        self.download()
-        self.process_annotations()
+        self._download()
+        self._process_annotations()
 
-        if self.features is not None:
-            self.load_features()
+        if self.features is not None and type(features) == str:
+            self._load_features()
+        elif self.features is not None and isinstance(self.features, type):
+            # object is given, do nothing.
+            pass
 
-        self.load()  # load questions and annotations
+        if load:
+            self._load()  # load questions and annotations
 
-        if self.has_annotations:
-            # This dictionnary will be used for evaluation
-            self.qid_to_annot = {a["question_id"]: a for a in self.annotations}
+            if self.has_annotations:
+                # This dictionnary will be used for evaluation
+                self.qid_to_annot = {a["question_id"]: a for a in self.annotations}
 
-        # aid_to_ans
-        self.ans_to_aid = {ans: i for i, ans in enumerate(self.answers)}
+            # aid_to_ans
+            self.ans_to_aid = {ans: i for i, ans in enumerate(self.answers)}
 
-    def load_questions(self, split):
+    def _load_questions(self, split):
         with open(self.path_questions[split]) as f:
             return json.load(f)["questions"]
 
-    def load_original_annotations(self, split):
+    def _load_original_annotations(self, split):
         with open(self.path_original_annotations[split]) as f:
             return json.load(f)["annotations"]
 
-    def load_processed_annotations(self, split):
+    def _load_processed_annotations(self, split):
         with open(self.path_annotations_processed[split]) as f:
             return json.load(f)
 
-    def load_features(self):
+    def _load_features(self):
         if self.split == "test":
             self.feats = get_features(
                 self.features, split="test2015", dir_data=self.dir_features,
@@ -182,30 +198,10 @@ class VQA(AbstractVQA):
             set((token for q in self.questions for token in tokenizer(q["question"])))
         )
 
-    def get_word_embeddings(self, name, freeze=True):
-        """
-        This will create the word embedding adapted to the dataset, and will cache it in the
-        dataset directory for faster usage.
-        """
-        os.makedirs(os.path.join(self.dir_dataset, "wordembeddings"), exist_ok=True)
-        path = os.path.join(self.dir_dataset, "wordembeddings", f"{name}.pth")
-        tokens = self.get_all_tokens()
-        if os.path.exists(path):
-            w_emb = WordEmbedding(tokens, dim=get_dim_from_name(name), freeze=freeze)
-            state_dict = torch.load(path)
-            w_emb.load_state_dict(state_dict)
-            print("done")
-        else:
-            print(
-                f"Loading and caching {name} word embeddings for {self.name} dataset."
-            )
-            w_emb = WordEmbedding.from_pretrained(
-                "glove.6B.300d", tokens=self.get_all_tokens(), dir_data=self.dir_data
-            )
-            torch.save(w_emb.state_dict(), path)
-        return w_emb
+    def get_all_questions(self):
+        return (q["question"] for q in self.questions)
 
-    def process_annotations(self):
+    def _process_annotations(self):
         """Process answers to create answer tokens,
         and precompute VQA score for faster evaluation.
         This follows the official VQA evaluation tool.
@@ -213,8 +209,8 @@ class VQA(AbstractVQA):
         path_train = self.path_annotations_processed["train"]
         path_val = self.path_annotations_processed["val"]
         if not os.path.exists(path_train) or not os.path.exists(path_val):
-            annotations_train = self.load_original_annotations("train")
-            annotations_val = self.load_original_annotations("val")
+            annotations_train = self._load_original_annotations("train")
+            annotations_val = self._load_original_annotations("val")
             all_annotations = annotations_train + annotations_val
 
             print("Processing annotations")
@@ -259,8 +255,8 @@ class VQA(AbstractVQA):
         #####################################
         if not os.path.exists(self.path_answers):
             print(f"Removing uncommon answers")
-            annotations_train = self.load_processed_annotations("train")
-            annotations_val = self.load_processed_annotations("val")
+            annotations_train = self._load_processed_annotations("train")
+            annotations_val = self._load_processed_annotations("val")
             all_annotations = annotations_train + annotations_val
 
             occ = Counter(annot["multiple_choice_answer"] for annot in all_annotations)
@@ -272,7 +268,7 @@ class VQA(AbstractVQA):
             with open(self.path_answers, "w") as f:
                 json.dump(self.answers, f)
 
-    def load(self):
+    def _load(self):
         print("Loading questions")
         with open(self.path_questions[self.split]) as f:
             self.questions = json.load(f)["questions"]
@@ -286,7 +282,7 @@ class VQA(AbstractVQA):
         with open(self.path_answers) as f:
             self.answers = json.load(f)
 
-    def download(self):
+    def _download(self):
         # download all splits
         for split in self.url_questions.keys():
             url_questions = self.url_questions[split]
@@ -305,6 +301,9 @@ class VQA(AbstractVQA):
                 download_and_unzip(url_annotations, directory=directory)
 
     def __len__(self):
+        """
+        Returns the number of (question-image-answer) items in the dataset.
+        """
         return len(self.questions)
 
     def __getitem__(self, index):
@@ -351,7 +350,10 @@ class VQA(AbstractVQA):
 
     @staticmethod
     def collate_fn(batch):
-        no_collate_keys = ["scores", "question_id", "question_tokens"]
+        """
+        Use this method to collate batches of data.
+        """
+        no_collate_keys = ["scores", "question_id", "question"]
         result_batch = {}
         for key in batch[0]:
             if key not in no_collate_keys:
@@ -362,7 +364,14 @@ class VQA(AbstractVQA):
 
     def evaluate(self, predictions):
         """
-        predictions: list of answer_id
+        Evaluates a list of predictions, according to the VQA evaluation protocol. See https://visualqa.org/evaluation.html.
+
+        Args:
+            predictions (list): List of dictionnaries containing ``question_id`` and ``answer`` keys. The answer must be specified 
+                as a string.
+
+        Returns:
+            A dict of floats containing scores for "overall", "yes/no", number", and "other" questions.
         """
         scores = {"overall": [], "yes/no": [], "number": [], "other": []}
         for p in predictions:
@@ -373,91 +382,34 @@ class VQA(AbstractVQA):
             ans_type = annot["answer_type"]
             scores["overall"].append(score)
             scores[ans_type].append(score)
-        return {key: mean(score_list) if len(score_list) else 0.0 for key, score_list in scores.items()}
-
-
-class VQADataModule(pl.LightningDataModule):
-
-    dataset = VQA
-
-    def __init__(
-        self,
-        dir_data: str,
-        min_ans_occ=8,
-        features=None,
-        tokenize_question=False,
-        label="multilabel",
-        batch_size=512,
-        num_workers=4,
-    ):
-        super().__init__()
-        self.dir_data = dir_data
-        self.label = label
-        self.features = features
-        self.min_ans_occ = min_ans_occ
-        self.batch_size = batch_size
-        self.num_workers = num_workers
-        self.tokenize_question = tokenize_question
-
-    def prepare_data(self):
-        self.train_dataset = self.dataset(
-            dir_data=self.dir_data,
-            split="train",
-            features=self.features,
-            min_ans_occ=self.min_ans_occ,
-            label=self.label,
-        )
-
-    def setup(self):
-        self.val_dataset = self.dataset(
-            dir_data=self.dir_data,
-            split="val",
-            features=self.features,
-            min_ans_occ=self.min_ans_occ,
-            label=self.label,
-            tokenize_questions=self.tokenize_question,
-        )
-
-    def train_dataloader(self):
-        return DataLoader(
-            self.train_dataset,
-            batch_size=self.batch_size,
-            shuffle=True,
-            collate_fn=self.dataset.collate_fn,
-            num_workers=self.num_workers,
-            pin_memory=True,
-        )
-
-    def val_dataloader(self):
-        return DataLoader(
-            self.val_dataset,
-            batch_size=self.batch_size,
-            shuffle=False,
-            collate_fn=self.dataset.collate_fn,
-            num_workers=self.num_workers,
-            pin_memory=True,
-        )
-
-    def test_dataloader(self):
-        dataset = self.dataset(
-            dir_data=self.dir_data,
-            split="test",
-            features=self.features,
-            min_ans_occ=self.min_ans_occ,
-            label=self.label,
-            tokenize_questions=self.tokenize_question,
-        )
-        return DataLoader(
-            dataset,
-            batch_size=self.batch_size,
-            shuffle=False,
-            collate_fn=self.dataset.collate_fn,
-            num_workers=self.num_workers,
-            pin_memory=True,
-        )
+        return {
+            key: mean(score_list) if len(score_list) else 0.0
+            for key, score_list in scores.items()
+        }
 
 
 class VQA2(VQA):
+    """
+    Pytorch Dataset implementation for the VQA v2 dataset (visual question answering). 
+    See https://visualqa.org/ for more details about it.
+    
+    When this class is instanciated, data will be downloaded in the directory specified by the ``dir_data`` parameter.
+    Pre-processing of questions and answers will take several minutes.
+
+    When the ``features`` argument is specified, visual features will be downloaded as well. About 60Go will be 
+    necessary for downloading and extracting features.
+
+    Args:
+        dir_data (str): dir for the multimodal cache (data will be downloaded in a vqa2/ folder inside this directory
+        features (str): which visual features should be used. Choices: ``coco-bottomup`` or ``coco-bottomup-36``
+        split (str): Which t [``train``, ``val``, ``test``]
+        dir_features (str): directory to download features. If None, defaults to $dir_data/features
+        label (str): either `multilabel`, or `best`. For `multilabel`, GT scores for questions are
+            given by the score they are assigned by the VQA evaluation. 
+            If `best`, GT is the label of the top answer.
+        tokenize_questions (bool): If True, preprocessing will tokenize questions into tokens.
+            The tokens are stored in item["question_tokens"].
+    """
 
     name = "vqa2"
 
@@ -484,11 +436,27 @@ class VQA2(VQA):
     }
 
 
-class VQA2DataModule(VQADataModule):
-    dataset = VQA2
-
-
 class VQACP(VQA):
+    """  Pytorch Dataset implementation for the VQA-CP v1 dataset (visual question answering). 
+    See https://www.cc.gatech.edu/grads/a/aagrawal307/vqa-cp/ for more details about it.
+    
+    When this class is instanciated, data will be downloaded in the directory specified by the ``dir_data`` parameter.
+    Pre-processing of questions and answers will take several minutes.
+
+    When the ``features`` argument is specified, visual features will be downloaded as well. About 60Go will be 
+    necessary for downloading and extracting features.
+
+    Args:
+        dir_data (str): dir for the multimodal cache (data will be downloaded in a vqa2/ folder inside this directory
+        features (str): which visual features should be used. Choices: ``coco-bottomup`` or ``coco-bottomup-36``
+        split (str): Which t [``train``, ``val``, ``test``]
+        dir_features (str): directory to download features. If None, defaults to $dir_data/features
+        label (str): either `multilabel`, or `best`. For `multilabel`, GT scores for questions are
+            given by the score they are assigned by the VQA evaluation. 
+            If `best`, GT is the label of the top answer.
+        tokenize_questions (bool): If True, preprocessing will tokenize questions into tokens.
+            The tokens are stored in item["question_tokens"].
+    """
 
     DOWNLOAD_SPLITS = ["train", "test"]
 
@@ -524,14 +492,27 @@ class VQACP(VQA):
             self.annotations = json.load(f)
 
 
-class VQACPDataModule(VQADataModule):
-    dataset = VQACP
-
-    def test_dataloader(self):
-        return None
-
-
 class VQACP2(VQACP):
+    """  Pytorch Dataset implementation for the VQA-CP v2 dataset (visual question answering). 
+    See https://www.cc.gatech.edu/grads/a/aagrawal307/vqa-cp/ for more details about it.
+    
+    When this class is instanciated, data will be downloaded in the directory specified by the ``dir_data`` parameter.
+    Pre-processing of questions and answers will take several minutes.
+
+    When the ``features`` argument is specified, visual features will be downloaded as well. About 60Go will be 
+    necessary for downloading and extracting features.
+
+    Args:
+        dir_data (str): dir for the multimodal cache (data will be downloaded in a vqa2/ folder inside this directory
+        features (str): which visual features should be used. Choices: ``coco-bottomup`` or ``coco-bottomup-36``
+        split (str): Which t [``train``, ``val``, ``test``]
+        dir_features (str): directory to download features. If None, defaults to $dir_data/features
+        label (str): either `multilabel`, or `best`. For `multilabel`, GT scores for questions are
+            given by the score they are assigned by the VQA evaluation. 
+            If `best`, GT is the label of the top answer.
+        tokenize_questions (bool): If True, preprocessing will tokenize questions into tokens.
+            The tokens are stored in item["question_tokens"].
+    """
 
     name = "vqacp2"
 
@@ -545,9 +526,3 @@ class VQACP2(VQACP):
         "test": "https://computing.ece.vt.edu/~aish/vqacp/vqacp_v2_test_annotations.json",
     }
 
-
-class VQACP2DataModule(VQADataModule):
-    dataset = VQACP2
-
-    def test_dataloader(self):
-        return None
